package gus05.entity.gus.app.feedback.gui.addnote;


import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Graphic;
import gus05.framework.features.Execute;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.BevelBorder;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Map;



public class GuiAddNote implements Entity, Graphic, Execute, ActionListener {


    public String getName()				{return "gus.app.feedback.gui.addnote";}
    public String getCreationDate()		{return "2009.05.29";}

    
    public static final String TYPE_CORRECTION = "correction";
    public static final String TYPE_EVOLUTION = "evolution";
    
    public static final Color COLOR_CORRECTION = Color.ORANGE;
    public static final Color COLOR_EVOLUTION = Color.GREEN;
    

    private Service registerNote;
    private Service persist;
    private Map prop;
    
    private JPanel panel;
    private JTextArea area;
    private JRadioButtonBold box_improve;
    private JRadioButtonBold box_bug;
    private JLabel labelInfo;
    private JButton buttonReset;
    



    public GuiAddNote() throws Exception
    {
    	registerNote = Outside.service(this,"gus.app.feedback.registernotes");
    	persist = Outside.service(this,"gus.graphic.persistence.persistence1");
    	prop = (Map) Outside.resource(this,"app.prop.map");
    	
    	box_improve = new JRadioButtonBold("Evolution",true);
    	box_bug = new JRadioButtonBold("Correction",false);
    	
    	box_improve.addActionListener(this);
    	box_bug.addActionListener(this);
    	
    	ButtonGroup group = new ButtonGroup();
    	group.add(box_improve);
    	group.add(box_bug);
    	
    	labelInfo = new JLabel(" ");
    	labelInfo.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
    	
    	area = new JTextArea();
    	area.setLineWrap(true);
    	area.setMargin(new Insets(3,3,3,3));
    	
    	buttonReset = new JButton("reset");
    	buttonReset.setBorder(new OvalBorder());
    	buttonReset.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {reset();}
		});
    	
    	
    	
    	JPanel p_top0 = new JPanel(new GridLayout(1,2));
    	p_top0.add(box_improve);
    	p_top0.add(box_bug);
    	
    	JPanel p_top = new JPanel(new BorderLayout());
    	p_top.add(p_top0,BorderLayout.CENTER);
    	p_top.add(buttonReset,BorderLayout.EAST);
    	
        panel = new JPanel(new BorderLayout());
        panel.add(p_top,BorderLayout.NORTH);
        panel.add(new JScrollPane(area),BorderLayout.CENTER);
        panel.add(labelInfo,BorderLayout.SOUTH);
        
        
        persist("area",area);
        persist("box_improve",box_improve);
        persist("box_bug",box_bug);
        
        updateArea();
    }
    
    



    public void execute() throws Exception
    {
    	String content = area.getText();
    	if(content.equals("")) return;
    	String type = getType();
    	
    	registerNote.register(type,content);
    	area.setText("");
    }
    
    
    
    
    private String getType()
    {return box_bug.isSelected()?TYPE_CORRECTION:TYPE_EVOLUTION;}

    
    
    private void persist(String key, JComponent comp) throws Exception
    {persist.register(getName()+"_"+key,comp);}
    
    
    
    
    private String prop(String key)
    {return prop.containsKey(key)?(String) prop.get(key):"?";}
    
    
    
    private String appGus05ID()
    {return prop("app.gus05id");}
    
    
    


    public JComponent gui()
    {return panel;}
    
    
    
	public void actionPerformed(ActionEvent e)
	{updateArea();}

	
	
	private void updateArea()
	{
		if(box_bug.isSelected())
		{
			area.setForeground(COLOR_CORRECTION);
			labelInfo.setText(" correction for: "+appGus05ID());
		}
		else
		{
			area.setForeground(COLOR_EVOLUTION);
			labelInfo.setText(" evolution for: "+appGus05ID());
		}
	}
	
	
	
	private void reset()
	{
		area.setText("");
	}
	

}

