package gus05.entity.gus.app.feedback.gui.viewnotes;


import java.awt.Color;
import java.awt.Component;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Graphic;

import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;



public class GuiViewNotes implements Entity, Graphic, ActionListener, TreeSelectionListener {


    public String getName()				{return "gus.app.feedback.gui.viewnotes";}
    public String getCreationDate()		{return "2009.05.30";}

    
    
    public static final String TYPE_CORRECTION = "correction";
    public static final String TYPE_EVOLUTION = "evolution";
    
    public static final Color COLOR_CORRECTION = Color.ORANGE;
    public static final Color COLOR_EVOLUTION = Color.GREEN;
    
    

    private Service registerNote;
    private Service treeTooltip;
    
    private JSplitPane split;
    private JTextArea area;
    
    private TreeModel0 model;
    private JTree tree;
    
    private Map map1;
    private Map map2;
    private ArrayList keys;



    public GuiViewNotes() throws Exception
    {
    	registerNote = Outside.service(this,"gus.app.feedback.registernotes");
    	treeTooltip = Outside.service(this,"gus.graphic.tree.tooltip");
    	
    	map2 = new HashMap();
    	keys = new ArrayList();
    	
    	area = new JTextArea();
    	area.setEditable(false);
    	area.setLineWrap(true);
    	area.setMargin(new Insets(3,3,3,3));
    	
    	tree = new JTree();
    	tree.setRootVisible(false);
    	tree.addTreeSelectionListener(this);
    	tree.setCellRenderer(new TreeCellRenderer1());
    	
    	treeTooltip.give(tree);
    	
        split = new JSplitPane();
        split.setDividerSize(3);
        split.setDividerLocation(140);
        split.setLeftComponent(new JScrollPane(tree));
        split.setRightComponent(new JScrollPane(area));
        
        registerNote.addActionListener(this);
        updateGui();
    }



    public JComponent gui()
    {return split;}
    
    
	public void actionPerformed(ActionEvent e)
	{updateGui();}

	
	
	
	private void updateGui()
	{
		try
		{
			map1 = (Map) registerNote.provide();
			map2.clear();
			
			
			Iterator it1 = map1.keySet().iterator();
			while(it1.hasNext())
			{
				String key = (String) it1.next();
				Map prop = (Map) map1.get(key);
				String appID = (String) prop.get(NOTES.KEY_APPGUS05ID);
				
				if(!map2.containsKey(appID))
					map2.put(appID,new ArrayList());
				ArrayList list = (ArrayList) map2.get(appID);
				list.add(key);
			}
			
			
			Iterator it2 = map2.keySet().iterator();
			while(it2.hasNext())
			{
				String key = (String) it2.next();
				ArrayList list = (ArrayList) map2.get(key);
				Collections.sort(list);
			}
			
			
			keys = new ArrayList(map2.keySet());
			Collections.sort(keys);
			tree.setModel(new TreeModel0());
		}
		catch(Exception e)
		{Outside.err(this,"updateGui()",e);}
	}
	
	
	
	
	
	
	
	public void valueChanged(TreeSelectionEvent e)
	{
		String s = (String) tree.getLastSelectedPathComponent();
		if(map1.containsKey(s))
		{
			Map prop = (Map) map1.get(s);
			Color color = findColor(prop);
			String content = findContent(prop);
			
			area.setText(content);
			area.setForeground(color);
		}
		else area.setText("");
	}

	
	
	
	
	private Color findColor(Map prop)
	{
		if(!prop.containsKey(NOTES.KEY_TYPE)) return Color.BLACK;
		String type = (String) prop.get(NOTES.KEY_TYPE);
		if(type.equals(TYPE_CORRECTION)) return COLOR_CORRECTION;
		if(type.equals(TYPE_EVOLUTION)) return COLOR_EVOLUTION;
		return Color.LIGHT_GRAY;
	}
	
	private String findContent(Map prop)
	{
		if(prop.containsKey(NOTES.KEY_CONTENT))
			return (String) prop.get(NOTES.KEY_CONTENT);
		return "";
	}
	
	
	
	
	
	private class TreeModel0 implements TreeModel
	{
		public static final String ROOT = "ROOT";
		public Object getRoot() {return ROOT;}

		
		public Object getChild(Object parent, int index)
		{
			if(parent.equals(ROOT)) return keys.get(index);
			if(map2.containsKey(parent))
			{
				ArrayList list = (ArrayList) map2.get(parent);
				return list.get(index);
			}
			return null;
		}

		public int getChildCount(Object parent)
		{
			if(parent.equals(ROOT)) return keys.size();
			if(map2.containsKey(parent))
			{
				ArrayList list = (ArrayList) map2.get(parent);
				return list.size();
			}
			return 0;
		}


		public int getIndexOfChild(Object parent, Object child)
		{
			if(parent.equals(ROOT)) return keys.indexOf(child);
			if(map2.containsKey(parent))
			{
				ArrayList list = (ArrayList) map2.get(parent);
				return list.indexOf(child);
			}
			return -1;
		}

		
		public boolean isLeaf(Object node)
		{return !node.equals(ROOT) && !keys.contains(node);}
		

		public void addTreeModelListener(TreeModelListener l) {}
		public void removeTreeModelListener(TreeModelListener l) {}
		public void valueForPathChanged(TreePath path, Object newValue) {}
	}
	
	
	
	
	
	
	
	
	

	private class TreeCellRenderer1 extends DefaultTreeCellRenderer
	{
		public Component getTreeCellRendererComponent(JTree tree,Object value,boolean sel,boolean expanded,boolean leaf,int row,boolean hasFocus)
		{
			super.getTreeCellRendererComponent(tree,value,sel,expanded,leaf,row,hasFocus);
			if(value==null)return this;
			
			setIcon(null);
			if(value instanceof ArrayList)
			{
				setForeground(Color.BLACK);
				return this;
			}
				
			if(value instanceof String)
			{
				String s = (String) value;
				if(map1.containsKey(s))
				{
					Map prop = (Map) map1.get(s);
					Color c = findColor(prop);
					setForeground(c);
				}
			}
			return this;
		}
	}




}

