package gus05.entity.gus.app.watchrelease.init;


import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.PrintStream;
import java.util.Properties;

import javax.swing.JOptionPane;



public class WatchReleaseInit implements Entity, Runnable, ActionListener {


    public String getName()				{return "gus.app.watchrelease.init";}
    public String getCreationDate()		{return "2009.02.15";}


    public static final String KEY_LATEST = "latest";
    public static final String PARAM_RELEASEINSTALLED = "releaseinstalled";
	
    
    private Service onlineWatching;
    private Service retrieveProp;
    private Service menuHolder;
    private Service checkFileName;
    
    private String[] args;
    private PrintStream out;
    
    private Properties prop;
    private boolean detected = false;
    private boolean running = false;

    
    
    
    
    public WatchReleaseInit() throws Exception
    {
    	onlineWatching = Outside.service(this,"gus.watching.online");
    	retrieveProp = Outside.service(this,"gus.app.watchrelease.getreleaseprop");
    	menuHolder = Outside.service(this,"gus.app.watchrelease.menu");
    	checkFileName = Outside.service(this,"gus.app.watchrelease.checkfilename");
    	
    	args = (String[]) Outside.resource(this,"app.args");
    	out = (PrintStream) Outside.resource(this,"out");
    	
    	checkArgs();
    	
    	start();
    	onlineWatching.addActionListener(this);
    	retrieveProp.addActionListener(this);
    }

    
    public void actionPerformed(ActionEvent e)
    {start();}
	
    
    
    private void start()
    {
    	try
    	{
    		if(detected || running) return;
    		if(!onlineWatching.filter(null)) return;
    		
    		running = true;
    		new Thread(this).start();
    	}
    	catch(Exception e)
    	{Outside.err(this,"start()",e);}
    }
    
    
    
    
	public void run()
	{perform();running = false;}
	
	
	
	
	private void perform()
	{
		try
		{
			prop = (Properties) retrieveProp.provide();
	    	if(prop==null || prop.isEmpty()) return;
	    	
	    	String timeStamp = prop.getProperty(KEY_LATEST);
			String fileName = prop.getProperty(timeStamp);
			
			out.println("latest online release: "+timeStamp+":"+fileName);
			
			if(isNotUpToDate(fileName))
			{
				out.println("WARNING: current app version is not up to date !");
				String[] infos = new String[]{timeStamp,fileName};
				menuHolder.give(infos);
				detected = true;
			}
			else out.println("current app version is up to date");
		}
		catch(Exception e)
		{Outside.err(this,"perform()",e);}
	}
	
	
	
	
	
	
	
	
	
	
	
    
    
    private boolean isNotUpToDate(String fileName) throws Exception
    {return checkFileName.filter(fileName);}
    
    
    
    
    
    private void checkArgs()
    {
    	for(int i=0;i<args.length;i++)
    	{
    		if(args[i].startsWith(PARAM_RELEASEINSTALLED+"="))
    		{
    			String installed = args[i].substring(PARAM_RELEASEINSTALLED.length()+1);
    			String message = "Release "+installed+" was successfuly installed";
        		JOptionPane.showMessageDialog(null,message,"Installation complete",JOptionPane.INFORMATION_MESSAGE);
        		return;
    		}
    	}
    }
    
}

