package gus05.entity.gus.app.watchrelease.installfile;


import java.io.File;
import java.io.PrintStream;
import java.util.Map;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Execute;
import gus05.framework.features.Give;



public class InstallFile implements Entity, Give {


    public String getName()				{return "gus.app.watchrelease.installfile";}
    public String getCreationDate()		{return "2009.04.26";}

    
    public static final String JARFILENAME = "installer.jar";
    public static final String POOL_OBJ_CONFIRMEXIT = "app.confirmexit";
    public static final String POOL_EXECUTE_BEFOREXIT = "app.executebeforeexit";
    public static final String PARAM_RELEASEINSTALLED = "releaseinstalled";
	
    
    private Service initJarTool;
    private Service launchApp;
	private Service exitApp;
	
	private PrintStream out;
	private String[] args;
    private Map pool;
    private File storeDir;
    
    private File newAppFile;
    private File installer;
    
    

    
    public InstallFile() throws Exception
    {
    	initJarTool = Outside.service(this,"gus.app.watchrelease.initjartool");
    	launchApp = Outside.service(this,"gus.java.launcher.launchapp");
    	exitApp = Outside.service(this,"app.executeexit");
    	
    	out = (PrintStream) Outside.resource(this,"out");
    	args = (String[]) Outside.resource(this,"app.args");
    	pool = (Map) Outside.resource(this,"app.internalmap");
    	storeDir = (File) Outside.resource(this,"defaultdir");
    }



    public void give(Object obj) throws Exception
    {
    	newAppFile = (File) obj;
    	
    	if(newAppFile==null)
    		throw new Exception("release file is null");
		if(!newAppFile.exists())
			throw new Exception("release file does not exist: "+newAppFile);
		
		installer = new File(storeDir,JARFILENAME);
		if(!installer.exists()) initJarTool.give(installer);
		
		if(!installer.exists())
			throw new Exception("jar tool file does not exist: "+installer);
		
		if(pool.containsKey(POOL_OBJ_CONFIRMEXIT))
			pool.remove(POOL_OBJ_CONFIRMEXIT);
		
		pool.put(POOL_EXECUTE_BEFOREXIT, new AppSelfInstall());
		
		out.println("execute tool to install new version and restart application ...");
		exitApp.execute();
    }


    
    
    
    private class AppSelfInstall implements Execute
	{
		public void execute() throws Exception
		{
			String intallerPath = installer.getAbsolutePath();
			String args = prepareArgs();
			
			launchApp.give(new String[]{intallerPath,null,args});
		}
	}
    
    
    
    
    
    private String prepareArgs()
	{
		String oldAppPath = findAppPath().getAbsolutePath();
		String newAppPath = newAppFile.getAbsolutePath();
		String newReleaseArg = PARAM_RELEASEINSTALLED+"="+newAppFile.getName();
		String previousArgs = previousArgs();
		
		return oldAppPath+";"+newAppPath+";"+newReleaseArg+previousArgs;
	}
    


    public static File findAppPath()
    {
    	String value = System.getProperty("java.class.path");
    	String[] paths = value.split(File.pathSeparator);
    	return new File(paths[0]);
    }
    
    
    
    private String previousArgs()
	{
		if(args.length==0) return "";
		StringBuffer b = new StringBuffer();
		for(int i=0;i<args.length;i++)
		{
			String a = args[i];
			if(!a.startsWith(PARAM_RELEASEINSTALLED+"="))
				b.append(a+";");
		}
		b.deleteCharAt(b.length()-1);
		return ";"+b.toString();
	}
}

