package gus05.entity.gus.data.editor.map.string.editor_a2;


import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.Map;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Give;
import gus05.framework.features.Graphic;
import gus05.framework.features.Register;
import gus05.framework.features.Transform;

import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.SwingConstants;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;



public class MapStringEditor_a2 implements Entity, Give, Graphic, Register, DocumentListener {


    public String getName()				{return "gus.data.editor.map.string.editor_a2";}
    public String getCreationDate()		{return "2006.10.10";}



    private Service textCompPanel;
    private Service factory;
    
    private Map map;
    
    
    private JSplitPane split;
    private JTextArea textArea;
    private JComponent textComp;
    
    private JList mapList;
    private JTextField mapTextField;
    private JLabel keyLabel;
    private JLabel mapLabel;
    
    private Icon icon;
    private Transform trans;
    
    private String selectedKey;
    private boolean initializingArea = false;
    private boolean textEdited = false;
    
    
    
    
    
    public MapStringEditor_a2() throws Exception
    {
        textCompPanel = Outside.service(this,"gus.graphic.textcomp.buildgui.panel1");
        factory = Outside.service(this,"gus.collection.map.guifactory");
        
        keyLabel = new JLabel(" ");
        keyLabel.setHorizontalAlignment(SwingConstants.CENTER);
        
        textArea = new JTextArea();
        textArea.getDocument().addDocumentListener(this);
        
        textComp = (JComponent) textCompPanel.transform(textArea);
        
        
        mapList = (JList) factory.retrieve("mapJList");
        mapTextField = (JTextField)factory.retrieve("mapJTextField");
        mapLabel = (JLabel)factory.retrieve("mapJLabel");
        
        mapList.getSelectionModel().addListSelectionListener(new ListSelectionListener(){
            public void valueChanged(ListSelectionEvent e)
            {selectEntry();}});
        
        mapTextField.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e)
            {updateEntry();}});
        
        mapTextField.addKeyListener(new KeyAdapter(){
            public void keyPressed(KeyEvent e)
            {if(e.getKeyCode()==KeyEvent.VK_F1) retrieveEntry();}
            });
        
        
        JPanel p_right = new JPanel(new BorderLayout());
        p_right.add(keyLabel,BorderLayout.NORTH);
        p_right.add(textComp,BorderLayout.CENTER);
        
        JPanel p_left = new JPanel(new BorderLayout());
        p_left.add(mapTextField,BorderLayout.NORTH);
        p_left.add(new JScrollPane(mapList),BorderLayout.CENTER);
        p_left.add(mapLabel,BorderLayout.SOUTH);
        
        
        split = new JSplitPane();
        split.setDividerSize(3);
        split.setDividerLocation(120);
        split.setOneTouchExpandable(false);
        split.setContinuousLayout(true);
                
        split.setLeftComponent(p_left);
        split.setRightComponent(p_right);
    }
    
    
    
    
    public void give(Object obj) throws Exception
    {
        map = (Map)obj;
        textArea.setText("");
        factory.give(map);
    }

    
    public void register(String key, Object obj) throws Exception
    {
        if(key.equals("icon"))
        {
            if(obj instanceof Icon)
            {registerIcon((Icon)obj);return;}
            if(obj instanceof Transform)
            {registerIconTrans((Transform)obj);return;}
        }
        throw new Exception("Unknown key: "+key);
    }
    
    
    public JComponent gui()
    {return split;}
    
    
    
    
    private void mapToArea()
    {
        initializingArea = true;
        String value = (String)map.get(selectedKey);
        textArea.setText(value);
        textArea.setCaretPosition(0);
        textSynchronized();
        initializingArea = false;
    }
    
    
    
    private void areaToMap()
    {
        String value = textArea.getText();
        map.put(selectedKey,value);
        textSynchronized();
    }
    
    
    
    
    private void changeSelectedKey(String newKey)
    {
        selectedKey = newKey;
        mapTextField.setText(selectedKey);
        keyLabel.setText(selectedKey);
        keyLabel.setIcon(findIcon(selectedKey));
    }
    
    
    
    
    
    
    
    
    
    private void selectEntry()
    {
        if(mapList.isSelectionEmpty())return;
        
        String newKey = (String)mapList.getSelectedValue();
        changeSelectedKey(newKey);
        mapToArea();
    }
    
    
    
    
    private void updateEntry()
    {
        String newKey = mapTextField.getText();
        if(newKey.equals(""))return;
        
        changeSelectedKey(newKey);
        areaToMap();
    }
    
    
    
    private void retrieveEntry()
    {
        String newKey = mapTextField.getText();
        if(!map.containsKey(newKey))return;
        
        changeSelectedKey(newKey);
        mapToArea();
    }
    
    



    


    
    
    
    
    
    
    public void changedUpdate(DocumentEvent e) {}
    public void insertUpdate(DocumentEvent e) {if(!initializingArea)textEdited();}
    public void removeUpdate(DocumentEvent e) {if(!initializingArea)textEdited();}
    
    
    private void textEdited()
    {
        textEdited = true;
        keyLabel.setFont(keyLabel.getFont().deriveFont(Font.ITALIC));
    }
    
    
    
    private void textSynchronized()
    {
        textEdited = false;
        keyLabel.setFont(keyLabel.getFont().deriveFont(Font.PLAIN));
    }
    
    
    
    
    
    
    
    
    
    
    
    
    private void registerIcon(Icon icon)
    {
        this.icon = icon;
        ListCellRenderer renderer = mapList.getCellRenderer();
        mapList.setCellRenderer(new ListRenderer0(renderer));
    }
    
    
    private void registerIconTrans(Transform trans)
    {
        this.trans = trans;
        ListCellRenderer renderer = mapList.getCellRenderer();
        mapList.setCellRenderer(new ListRenderer0(renderer));
    }
    
    
    
    
    private Icon findIcon(String key)
    {
        try
        {
            if(trans==null) return icon;
            return (Icon) trans.transform(key);
        }
        catch(Exception e)
        {Outside.err(this,"findIcon(String)",e);}
        return null;
    }
    
    
    
    
    private class ListRenderer0 implements ListCellRenderer
    {
        private ListCellRenderer renderer;
        
        public ListRenderer0(ListCellRenderer renderer)
        {this.renderer = renderer;}
        
        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
        {
            JLabel label = (JLabel)renderer.getListCellRendererComponent(list,value,index,isSelected,cellHasFocus);
            Icon icon = findIcon((String)value);
            label.setIcon(icon);
            return label;
        }
    }
   
    
}

