package gus05.entity.gus.file.editor.hexa.editor2;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Give;
import gus05.framework.features.Graphic;
import gus05.framework.features.Register;
import gus05.framework.tools.DefaultSupport;


public class HexaEditor2 extends DefaultSupport implements Entity, Graphic, Give, Register, ActionListener
{

    public String getName()				{return "gus.file.editor.hexa.editor2";}
    public String getCreationDate()		{return "2007.03.02";}
    
    
    public final static int LIMIT = 5000;
    

    public Service selectionSupport;
    
    private JPanel panel;
    private JTextArea textArea;
    private JLabel label;
    
    
    private File file;


    public HexaEditor2() throws Exception
    {
        selectionSupport = Outside.service(this,"gus.graphic.textcomp.selection.support1");
        
        textArea = new JTextArea();
        textArea.setWrapStyleWord(true);
        textArea.setLineWrap(true);
        textArea.setFont(new Font("Courier New", Font.PLAIN, 12));
        textArea.setEditable(false);
        
        label = new JLabel(" ");
        
        selectionSupport.give(textArea);
        selectionSupport.addActionListener(this);
        
        panel = new JPanel(new BorderLayout());
        panel.add(new JScrollPane(textArea),BorderLayout.CENTER);
        panel.add(label,BorderLayout.SOUTH);
    }





    public JComponent gui()
    {return panel;}


    
    

    public void give(Object obj) throws Exception
    {
        file = (File)obj;
        
        byte[] data = loadByteArray();
        String hexa = convertFromByteArray(data);
        textArea.setText(hexa);
        textArea.moveCaretPosition(0);
    }
    
    
    
    public void actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if(command.equals("start()")) start();
        if(command.equals("stop()")) stop();
    }
    
    
    
    
    
    private void start()
    {label.setText(" ");}
    
    private void stop()
    {
        String selected = textArea.getSelectedText();
        if(selected==null) selected = "";
        selected = selected.replace(" ","");
        
        if(selected.equals(""))
        {label.setText(" ");return;}
        
        if(selected.length()%2==1) selected = selected.substring(0,selected.length()-1);
        
        int number = selected.length()/2;
        
        
        StringBuffer buff = new StringBuffer(selected+" ["+number+"]");
        
        try
        {
            byte n = Byte.parseByte(selected,16);
            buff.append(" byte="+(int)n);
        }
        catch(Exception e){}
        
        try
        {
            short n = Short.parseShort(selected,16);
            buff.append(" short="+(int)n);
        }
        catch(Exception e){}
        
        try
        {
            int n = Integer.parseInt(selected,16);
            buff.append(" integer="+n);
            buff.append(" char="+(char)n);
        }
        catch(Exception e){}
        
        try
        {
            long n = Long.parseLong(selected,16);
            buff.append(" long="+n);
        }
        catch(Exception e){}
        
//        try
//        {
//            int charNumber = number/2;
//            buff.append(" text=");
//            for(int i=0;i<charNumber;i++)
//            {
//                int n = Integer.parseInt(selected.substring(4*i,4*i+4),16);
//                buff.append((char)n);
//            }
//        }
//        catch(Exception e){}
        
        
        label.setText(buff.toString());
    }
    
    
    
    
    
    
    
    
    private byte[] loadByteArray() throws IOException
    {
        int arrayLength = Math.min((int)file.length(),LIMIT);
        FileInputStream fis = new FileInputStream(file);
        byte[] data = new byte[arrayLength];
        fis.read(data);
        fis.close();
        
        return data;
    }
    
    
    
    
    
    
    private String convertFromByteArray(byte[] array)
    {
        StringBuffer b = new StringBuffer(array.length*2);
        for(int i=0;i<array.length;i++)
        {
            b.append(bitsToChar(array[i] >> 4));
            b.append(bitsToChar(array[i] & 0x0000000F));
            b.append(" ");
        }
        return b.toString();
    }
    
    
    private char bitsToChar(int bits)
    {
        int j = bits & 0x0000000F;
        switch (j) {
        	case 0: return '0';
        	case 1: return '1';
        	case 2: return '2';
        	case 3: return '3';
        	case 4: return '4';
        	case 5: return '5';
        	case 6: return '6';
        	case 7: return '7';
        	case 8: return '8';
        	case 9: return '9';
        	case 10: return 'A';
        	case 11: return 'B';
        	case 12: return 'C';
        	case 13: return 'D';
        	case 14: return 'E';
        	case 15: return 'F';
        	default: throw new IllegalArgumentException(j+" is not a valid value for hexadecimal digit.");
        }
    }
    
    
    
    
    
    public void register(String key, Object obj) throws Exception {
    }
    
    
}
