
package gus05.entity.gus.file.explorer.tree2;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Give;
import gus05.framework.features.Graphic;
import gus05.framework.features.Provide;
import gus05.framework.features.Retrieve;
import gus05.framework.resources.IconProvider;
import gus05.framework.tools.DefaultSupport;

import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JTree;
import javax.swing.filechooser.FileSystemView;
import javax.swing.plaf.basic.BasicTreeUI;
import javax.swing.tree.DefaultTreeCellRenderer;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.*;
import java.util.Map;


/**
 * @author gus
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class ExplorerTree2 extends DefaultSupport implements Entity, Give, Provide, Retrieve, Graphic, KeyListener {

    public String getName()				{return "gus.file.explorer.tree2";}
    public String getCreationDate()		{return "2006.09.08";}
    
    
    public static final Color COLOR_NORMAL = Color.BLUE;
    public static final Color COLOR_TOO_BIG = Color.GREEN;
    public static final Color COLOR_EMPTY = Color.RED;
    public static final Color COLOR_MULTIPLE = Color.ORANGE;
    public static final Color COLOR_REGISTEREDFILE = new Color(204,255,255);
    public static final Color COLOR_AUTOSAVEDFILE = new Color(204,204,255);
	
    
    private Service fileIcon;
    private Service handlerDigest;
    private Service popupMenu;
    private IconProvider iconProvider;
    
    private Map registeredFiles;
    private Map autosavedFiles;
    
    
	private ExplorerJTree tree;
	
	
	private Icon icon(String key)
    {return iconProvider.getIcon(key);}
	
	private Icon fileIcon(File file)
	{
	    try{return (Icon)fileIcon.transform(file);}
        catch(Exception e){return null;}
	}
	
	
	
	
	public ExplorerTree2() throws Exception
	{
	    fileIcon = Outside.service(this,"gus.icondisplay.file1");
	    handlerDigest = Outside.service(this,"gus.file.dirscan.info.handler.digest1");
        popupMenu = Outside.service(this,"gus.file.explorer.tree.popupmenu");
        iconProvider = (IconProvider)Outside.resource(this,"app.gui.icons");
        
        registeredFiles = (Map) Outside.resource(this,"app.filemap");
        autosavedFiles = (Map) Outside.resource(this,"app.datastore.filemap");
        
        
	    
		tree = new ExplorerJTree(this);
		tree.addKeyListener(this);
		tree.setCellRenderer(new ExplorerTreeCellRenderer());
		
		initDnD();
		initAppearance();
        popupMenu.give(tree);
	}

	
	
	
	public JComponent gui() 
	{return tree;}

	public void give(Object value) throws Exception
	{
	    if(value==null)tree.moveRoot();
	    else tree.setRoot((File)value);
	}
	
	private void refresh()
	{
	    tree.setRoot(tree.getRoot());
	}
	
	

	
	public Object provide() throws Exception
    {return tree.getSelectedFile();}
   
	
	
	public Object retrieve(String key) throws Exception
	{
	    if(key.equals("tree")) return tree;
        if(key.equals("keys"))
            return new String[]{"tree"};
        throw new Exception("Unknown key: "+key);
    }
	
	
	
	
	private void initDnD()
	{
	    try
        {
	        Service dnd = Outside.service(this,"gus.graphic.dnd");
	        dnd.register("give",null);
	        dnd.register("provide",this);
	        dnd.register("comp",tree);
        }
        catch(Exception e)
        {Outside.err(this,"initDnD()",e);}
	}
	
	
	
	
	private void initAppearance()
	{
	    try
        {
	        Icon collapsed = iconProvider.getIcon("TREE_collapsedState");
	        Icon expanded = iconProvider.getIcon("TREE_expandedState");
	        
	        if(collapsed!=null)((BasicTreeUI) tree.getUI()).setCollapsedIcon(collapsed);
	        if(expanded!=null)((BasicTreeUI) tree.getUI()).setExpandedIcon(expanded);
        }
        catch(Exception e)
        {Outside.err(this,"initAppearance()",e);}
	}
	
	
	
	
	
	
	private boolean isMultipleMDFile(File f)
	{
	    try{return handlerDigest.filter(f);}
	    catch(Exception e){return false;}
	}
	
	
	
	
	
	
	
	
	
	private class ExplorerTreeCellRenderer extends DefaultTreeCellRenderer {
	    
	    public Component getTreeCellRendererComponent(JTree tree,Object value,boolean sel,boolean expanded,boolean leaf,int row,boolean hasFocus)
	    {
	        super.getTreeCellRendererComponent(tree,value,sel,expanded,leaf,row,hasFocus);
	        if(value==null)return this; 
	        if(!(value instanceof File))return this;
	        
	        File file = (File)value;
	        if(!file.exists())return this;
	        
	        if(!file.canWrite()) setFont(getFont().deriveFont(Font.ITALIC));
	        else setFont(getFont().deriveFont(Font.PLAIN));
	        
	        setForeground(COLOR_NORMAL);
	        
	        if(autosavedFiles.values().contains(file))
                setBackgroundNonSelectionColor(COLOR_AUTOSAVEDFILE);
	        else if(registeredFiles.values().contains(file))
                setBackgroundNonSelectionColor(COLOR_REGISTEREDFILE);
	        else setBackgroundNonSelectionColor(tree.getBackground());
	        
	        if(isDriver(file)) customizeDriver(file);
	        else if(leaf) customizeLeaf(file);	
	        else customizeNode(file,expanded);  
	        return this;
	    }
	    
	    
	    
	    private void customizeLeaf(File file)
	    {
	        setText(file.getName());
	        setIcon(fileIcon(file));
	        
	        if(file.isDirectory())
	            setForeground(COLOR_EMPTY);
	        else
	        {
	            if(isMultipleMDFile(file))setForeground(COLOR_MULTIPLE);
	            else if(file.length()==0)setForeground(COLOR_EMPTY);
	        }
	    }
	    
	    
	    
	    private void customizeNode(File file, boolean expanded)
	    {
	        setText(file.getName());
	        if(expanded) setIcon(icon("FILE_dir2_"));
	        else setIcon(icon("FILE_dir_"));
	        
	        if(!containsFiles(file))setForeground(COLOR_EMPTY);
	        else if(containsTooManyFiles(file))
	        {
	            setForeground(COLOR_TOO_BIG);
	            setText(file.getName()+": TOO MANY FILES ("+number(file)+")");
	        }
	    }
	    
	    
	    
	    private void customizeDriver(File file)
	    {
	        setText(file.getAbsolutePath());
	        setIcon(icon("FILE_driver_"));
	    }
	    
	    
	     
	    
	    private boolean containsFiles(File file)
	    {
	        File[] children = file.listFiles();
	        if(children==null)return false;
	        
	        int n = children.length;
	         
	        for(int i=0;i<n;i++)
	        {
	            if(!children[i].isDirectory())return true;
	            if(containsFiles(children[i]))return true;
	        }
	        return false;
	    }
	    
	    
	    private boolean containsTooManyFiles(File file)
	    {return number(file)>ExplorerTreeModel.NB_LIMIT;}
    
	    private int number(File file)
	    {return file.listFiles().length;}
	    
	    private boolean isDriver(File file)
	    {return FileSystemView.getFileSystemView().isDrive(file);}
	}









    public void keyTyped(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {}
    public void keyPressed(KeyEvent e)
    {
        int key = e.getKeyCode();
        if(key==KeyEvent.VK_F5) refresh(); 
    }
    



    
}
