package gus05.entity.gus.graphic.textcomp.action.w.quickreplace;


import java.awt.Event;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.Action;
import javax.swing.KeyStroke;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.JTextComponent;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.features.Execute;
import gus05.framework.features.Give;
import gus05.framework.features.Transform;
import gus05.framework.resources.ActionBuilder;



public class QuickReplace implements Entity, Give, Transform {


    public String getName()				{return "gus.graphic.textcomp.action.w.quickreplace";}
    public String getCreationDate()		{return "2006.09.25";}



    private ActionBuilder actionBuilder;
    private KeyStroke quickKey;
    
    
    public QuickReplace() throws Exception
    {
        actionBuilder = (ActionBuilder) Outside.resource(this,"app.actionbuilder");
        quickKey = KeyStroke.getKeyStroke(KeyEvent.VK_W,Event.CTRL_MASK);
    }


    
    

    public void give(Object obj) throws Exception
    {
        JTextComponent comp = (JTextComponent)obj;
        Quick quick = new Quick(comp);
        Action action = actionBuilder.build("TEXTCOMP_quickreplace",quick);
        comp.getInputMap().put(quickKey, action);
    }



    public Object transform(Object obj) throws Exception
    {
        Quick quick = new Quick((JTextComponent)obj);
        return actionBuilder.build("TEXTCOMP_quickreplace",quick);
    }


    
    
    
    
    
    
    private String convert(String in)
    {
        StringBuffer b = new StringBuffer();
        for(int i=0;i<in.length();i++)
        {
            char c = in.charAt(i);
            if(c=='*' || c=='?' || c=='+' || c=='.' || c=='\\'
                || c=='(' || c==')'
                || c==']' || c=='['
                || c=='{' || c=='}'
            )
                b.append('\\');
            b.append(c);
        }
        return b.toString();
    }
    
    
    
    
    
    
    
    
    
    
    
    private class Quick implements Execute, DocumentListener, CaretListener, KeyListener, MouseMotionListener, MouseListener
    {
        private JTextComponent comp;
        
        private StringBuffer replacement;
        private int deleted_after;
        private int deleted_before;
        private String selected = "";
        private String textMem = "";
        private int position;
        private int lastPressed = -1;

        
        public Quick(JTextComponent comp)
        {
            this.comp = comp;
            
            comp.addKeyListener(this);
            comp.addMouseListener(this);
            comp.addMouseMotionListener(this);
            comp.addCaretListener(this);
            comp.getDocument().addDocumentListener(this);
            
            initReplacement();
        }
        
        
        
        private boolean selectedEmpty()
        {return selected.equals("");}
        
        private boolean selectionEmpty()
        {return comp.getSelectedText()==null;}
        
        
        
        private void initReplacement()
        {
            replacement = new StringBuffer();
            deleted_before = 0;
            deleted_after = 0;
        }
        
        
        private void initSelected()
        {
            selected = "";
            textMem = comp.getText();
        }
        
        
        private void initPosition()
        {position = comp.getCaretPosition();}
        
       
        
        private void init()
        {
            initSelected();
            initReplacement();
            initPosition();
        }
        
        
        
        
        private void setSelected(String selected)
        {
            this.selected = selected;
            replacement = new StringBuffer(); 
        }
        
        
        
        private void appendToReplacement(String inserted)
        {
            if(inserted.equals(comp.getText()) && inserted.length()>1)
                replacement = new StringBuffer();
            else replacement.append(inserted);
            deleted_before = 0; 
        }
        
        
        
        
        private void back()
        {
            int n = replacement.length(); 
            if(n>0) replacement.delete(n-1,n);
            else deleted_before+=1; 
        }
        
        
        private void suppr()
        {deleted_after++;}
        
        
        
        
        private void captureSelection()
        {
            String val = comp.getSelectedText();
            if(val==null)return;
            if(val.equals(""))return;
            setSelected(val);
        }
        
        

        private void caretMoved()
        {
            init();
            captureSelection();
        }
        
        
        
        
        public void changedUpdate(DocumentEvent e) {}
        
        public void keyTyped(KeyEvent e) {}
        public void keyPressed(KeyEvent e) {lastPressed=e.getKeyCode();}
        public void keyReleased(KeyEvent e) {}

        public void mouseDragged(MouseEvent e) {captureSelection();}
        public void mouseMoved(MouseEvent e) {}

        public void mouseClicked(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {caretMoved();}
        public void mouseReleased(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        
        
        
        public void insertUpdate(DocumentEvent e)
        {
            int start = e.getOffset();
            int length = e.getLength();
            try
            {
               String insertedText = e.getDocument().getText(start,length);
               appendToReplacement(insertedText);
            }
            catch(Exception ex){}
        }
        
        public void removeUpdate(DocumentEvent e)
        {
            if(e.getLength()>1) initReplacement();
            else if(lastPressed==8 && selectionEmpty()) back();
            else if(lastPressed==127 && selectionEmpty()) suppr();
            
            lastPressed = -1;
        }
        
        
        public void caretUpdate(CaretEvent evt)
        {
            // arrows
            if(lastPressed==37) caretMoved();
            if(lastPressed==38) caretMoved();
            if(lastPressed==39) caretMoved();
            if(lastPressed==40) caretMoved();
            
            lastPressed = -1;
        }
        
        
        
        
        public void execute() throws Exception
        {
            if(selectedEmpty()) replaceByPosition();
            else replaceByRegex();
            init();
        }
        
        
        
        
        private void replaceByPosition()
        {
            // modify at end of line
            if(position==textMem.length() || textMem.charAt(position)=='\n')
            {
                Transform t = new ReplaceAtEndOfLine(
                        replacement.toString(),
                        deleted_before,
                        deleted_after);
                
                executeTransform(t);
            }
            
            // modify at fixed position
            else
            {
                int d = 0;
                while(d<position && textMem.charAt(position-d-1)!='\n') d++;
                
                Transform t = new ReplaceAtFixedPosition(
                        replacement.toString(),
                        deleted_before,
                        deleted_after,
                        d);
                
                executeTransform(t);
            }
        }
        
        
        
        
        
        
        
        private void replaceByRegex()
        {
            // build the search pattern
            StringBuffer b = new StringBuffer();
            for(int i=0;i<deleted_before;i++)b.append(".");
            b.append(convert(selected));
            for(int i=0;i<deleted_after;i++)b.append(".");
            
            String exp = b.toString();
            String newExp = convert(replacement.toString());
            
            Transform t = new ReplaceByRegex(exp,newExp); 
            executeTransform(t);
        }
        
        
        private void executeTransform(Transform t)
        {
            try
            {
                comp.setText((String)t.transform(textMem));
                comp.requestFocus();
            }
            catch(Exception e)
            {Outside.err(QuickReplace.this,"executeTransform(Transform)",e);}
        }
    }
}

