package gus05.entity.gus.japanese.typingholder;


import javax.swing.SwingUtilities;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.JTextComponent;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Execute;
import gus05.framework.features.Transform;



public class TypingHolder implements Entity, Transform {


    public String getName()				{return "gus.japanese.typingholder";}
    public String getCreationDate()		{return "2006.11.10";}



    private Service hiraganaBuilder;
    private Service katakanaBuilder;
    

    public TypingHolder() throws Exception
    {
        hiraganaBuilder = Outside.service(this,"gus.japanese.hiragana.builder");
        katakanaBuilder = Outside.service(this,"gus.japanese.katakana.builder");
    }

    
    private String toHiragana(String text)
    {
        try{return (String) hiraganaBuilder.transform(text);}
        catch(Exception e){Outside.err(this,"toHiragana(String)",e);}
        return "";
    }
    
    private String toKatakana(String text)
    {
        try{return (String) katakanaBuilder.transform(text);}
        catch(Exception e){Outside.err(this,"toKatakana(String)",e);}
        return "";
    }
    
    
    

    
 
    public Object transform(Object obj) throws Exception
    {
        JTextComponent comp = (JTextComponent) obj;
        return new Holder(comp);
    }


    
    
    
    
    
    
    
    
    private class Holder implements Execute, DocumentListener, CaretListener
    {
        private JTextComponent comp;
        private boolean activated = false;
        
        
        private int caretPosition0;
        private StringBuffer buffer;
        private String input;
        private String replacement;
        
        private boolean textTyped = false;
        private boolean insertingJap = false;
        private boolean willResetLater = false;
        
        
        
        public Holder(JTextComponent comp)
        {this.comp = comp;}

        
        
        public void execute() throws Exception
        {
            if(activated) disactivate();
            else activate();
        }
                
        private void activate()
        {
            comp.getDocument().addDocumentListener(this);
            comp.addCaretListener(this);
            reset();
            activated = true;
        }
        
        private void disactivate()
        {
            comp.getDocument().removeDocumentListener(this);
            comp.removeCaretListener(this);
            activated = false;
        }
        
        private void reset()
        {
            buffer = new StringBuffer();
            caretPosition0 = comp.getCaretPosition();
            willResetLater = false;
        }
        
        
        
        
        private boolean isLowerCase(String text)
        {return text.toLowerCase().equals(text);}
        
        private boolean isUpperCase(String text)
        {return text.toUpperCase().equals(text);}
        
        
        private boolean isValidReplacement()
        {
            if(replacement.equals(input)) return false;
            if(input.endsWith("nn")) return true;
            if(input.endsWith("n")) return false;
            if(replacement.charAt(replacement.length()-1)==input.charAt(input.length()-1)) return false;
            return true;
        }
        

        
        public void changedUpdate(DocumentEvent e) {}
        public void insertUpdate(DocumentEvent e) {inserted(e.getLength(),e.getOffset());}
        public void removeUpdate(DocumentEvent e) {removed(e.getLength());}
        
        
        
        public void caretUpdate(CaretEvent e)
        {
            if(!textTyped) reset();
            textTyped = false;
        }
        
        
        
        
        
        private void removed(int length)
        {
            textTyped = true;
            if(insertingJap)return;
            
            int l = buffer.length();
            if(length<=l)buffer.delete(l-length,l);
            else
            {
                willResetLater = true;
                SwingUtilities.invokeLater(new ResetLater());
            }
        }
        
        
        
        
        
        
        private void inserted(int length, int offset)
        {
            try
            {
                willResetLater = false;
                
                if(insertingJap)return;
                if(length>1){reset();return;}
                
                textTyped = true;
                String s = comp.getDocument().getText(offset,1);
                
                buffer.append(s);
                input = buffer.toString();
                
                if(input.equals("-"))
                {
                    replacement = toKatakana(input);
                    SwingUtilities.invokeLater(new InsertLater());
                }
                else if(isLowerCase(input))
                {
                    replacement = toHiragana(input);
                    if(!isValidReplacement())return;
                    SwingUtilities.invokeLater(new InsertLater());
                }
                else if(isUpperCase(input))
                {
                    input = input.toLowerCase();
                    replacement = toKatakana(input);
                    if(!isValidReplacement())return;
                    SwingUtilities.invokeLater(new InsertLater());
                }
                else
                {
                    willResetLater = true;
                    SwingUtilities.invokeLater(new ResetLater());
                }
            }
            catch(Exception e)
            {Outside.err(TypingHolder.this,"inserted(length)",e);}
        }
        
        
        
        
        
        
        private class InsertLater implements Runnable
        {
            public void run()
            {
                try
                {
                    if(comp.getDocument().getLength()<caretPosition0) return;
                    
                    insertingJap = true;
                    comp.getDocument().remove(caretPosition0,input.length());
                    comp.getDocument().insertString(caretPosition0,replacement,null);
                    reset();
                    insertingJap = false;
                }
                catch(Exception e)
                {Outside.err(TypingHolder.this,"run()",e);}
            }
        }
        
        
        
        
        
        
        private class ResetLater implements Runnable
        {
            public void run()
            {if(willResetLater)reset();}
        }
        
        
    }

    
    
}

