package gus05.entity.gus.java.launcher.launchapp;


import java.io.File;
import java.io.PrintStream;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.features.Give;



public class LaunchApp implements Entity, Give {


    public String getName()				{return "gus.java.launcher.launchapp";}
    public String getCreationDate()		{return "2009.02.14";}


    private PrintStream out;
	
	public LaunchApp() throws Exception
	{out = (PrintStream) Outside.resource(this,"out");}
	
	
	
	
	public void give(Object obj) throws Exception
	{
		String[] info = (String[]) obj;
		if(info.length!=3) throw new Exception("Wrong info number: "+info.length);
		
		String appPath = info[0];
		String mainClass = info[1];
		String args = info[2];
		
		if(isDir(appPath))
			launchFromDir(appPath,mainClass,args);
		else if(isJar(appPath))
			launchFromJar(appPath,args);
		else throw new Exception("Invalid app path: "+appPath);
	}
	
	
	
	
	
	private void launchFromDir(String appDirPath, String mainClass, String args) throws Exception
	{
		if(appDirPath.contains(" "))
			appDirPath = "\""+appDirPath+"\"";
		
		String cmd = "java -classpath "+appDirPath+" "+mainClass;
		if(!empty(args)) cmd = cmd+" "+args;
		launch(cmd);
	}




	private void launchFromJar(String appJarPath, String args) throws Exception
	{
		if(appJarPath.contains(" "))
			appJarPath = "\""+appJarPath+"\"";
		
		String cmd = "java -jar "+appJarPath;
		if(!empty(args)) cmd = cmd+" "+args;
		launch(cmd);
	}
	
	
	
	
	
	
	private void launch(String cmd) throws Exception
	{
		out.println("launch "+cmd);
		File userDir = new File(System.getProperty("user.dir"));
		Runtime.getRuntime().exec(cmd,null,userDir);
	}

    
    
    private boolean isJar(String path)
    {
    	File f = new File(path);
    	return f.isFile() && f.getName().toLowerCase().endsWith(".jar");
    }
	
    private boolean isDir(String path)
    {
    	File f = new File(path);
    	return f.isDirectory();
    }
    
    
    private boolean empty(String value)
    {return value==null || value.equals("");}
	
}

