package gus05.manager.gus.endorf.layer3.loading;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.text.NumberFormat;
import java.util.Properties;
import java.util.Set;

import gus05.manager.gus.endorf.layer3.system1.System1;

public abstract class Loader {

    public final static String INSIDE_LOCATION = "/gus05/manager/gus/endorf/resources/";
    
    public final static URL insideURL(String id)
    {return Loader.class.getResource(INSIDE_LOCATION+id);}
    
    public final static InputStream insideStream(String id)
    {return Loader.class.getResourceAsStream(INSIDE_LOCATION+id);}
    
    
    
    private String name;
    private int number = 0;
    
    
    public Loader(String name)
    {
        this.name = name;
        load(); 
    }
    
    private void load()
    {
        debug("load()","LOADING "+name.toUpperCase());
        insideLoad();
        outsideLoad();
        debug("load()",number()+" "+name+" have been loaded");
    }
    
    
    protected abstract void insideLoad();
    protected abstract void outsideLoad();
    
    
    
    
    
    
    
    protected void debug(String id, String message)
    {System1.printDebug(this,id,message);}
    
    protected void warning(String id, String message)
    {System1.printWarning(this,id,message);}
    
    
    
    protected int number() {return number;}
    protected void increase() {number++;}
    protected void increase(int n) {number+=n;}
    
    
    
    
    protected Properties insideProp(String id) throws IOException
    {
        InputStream is = insideStream(id);
        if(is==null) {debug("insideProp(String)","resource not found: "+INSIDE_LOCATION+id);return null;}
        
        Properties p = new Properties();
        p.load(is);
        is.close();
        return p;
    }
    
    
    
    
    protected String insideTxt(String id) throws IOException
    {
        InputStream is = insideStream(id);
        if(is==null) {debug("insideTxt(String)","resource not found: "+INSIDE_LOCATION+id);return null;}
        
        StringBuffer b = new StringBuffer();
        int c;
        while((c=is.read())!=-1)
        b.append((char)c);
        is.close();
        return b.toString();
    }
    
    
    
    
    
    
    
    protected Set insideDir(String id) throws IOException
    {
        Properties p = insideProp(id+"/listing");
        if(p==null)return null;
        return p.keySet();
    }
    
    
    
    
    
    
    
    
    
    
    protected File outsideDir(String id)
    {
        File dir = System1.getFile(id);
        return outsideDir(dir,id);
    }
    
    
    protected File outsideDir(File dir, String id)
    {
        if(dir==null)
        {debug("outsideDir(String)",name+" dir is not available: "+id);return null;}
        System1.fileManager.initDir(dir);
        if(!dir.exists())
        {debug("outsideDir(String)",name+" dir "+dir.getAbsolutePath()+" does not exist");return null;}
        if(!dir.isDirectory())
        {debug("outsideDir(String)",name+" dir "+dir.getAbsolutePath()+" is not a directory");return null;}
        return dir;
    }
    
    
    
    
    protected File outsideFile(String id)
    {
        File file = System1.getFile(id);
        return outsideFile(file,id);
    }
    
    
    protected File outsideFile(File file, String id)
    {
        if(file==null)
        {debug("outsideFile(String)",name+" file is not available: "+id);return null;}
        System1.fileManager.initFile(file);
        if(!file.exists())
        {debug("outsideFile(String)",name+" file "+file.getAbsolutePath()+" does not exist");return null;}
        if(!file.isFile())
        {debug("outsideFile(String)",name+" file "+file.getAbsolutePath()+" is not a file");return null;}
        return file;
    }
    
    
    
    
    
    public static NumberFormat numberFormat = NumberFormat.getInstance();
    public static String ns(long value) 			{return numberFormat.format(value)+" ns";}
    
}
